/**********************************************************************
Copyright (c) 2006 Travis Reeder and others. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

Contributors:
    ...
**********************************************************************/
package org.datanucleus.sql4o.jdbc;

import com.db4o.ObjectContainer;
import com.db4o.Db4o;
import com.db4o.ext.DatabaseFileLockedException;

import java.sql.*;
import java.util.Properties;

/**
 * JDBC driver for db4o.
 * Accepts URLs of the form
 * <ul>
 * <li>jdbc:db4o:http://host:port</li>
 * <li>jdbc:db4o:file:{filename}</li>
 * </ul>
 */
public class Db4oDriver implements Driver
{
    public static final int VERSION_MAJOR = 1;

    public static final int VERSION_MINOR = 0;

    private static final String DRIVER_NAME = "db4o JDBC Driver";

    static
    {
        try
        {
            // Register the JWDriver with DriverManager
            Db4oDriver driver = new Db4oDriver();
            DriverManager.registerDriver(driver);
            // System.setSecurityManager(new RMISecurityManager());
        }
        catch (SQLException e)
        {
            e.printStackTrace();
        }
    }

    public Db4oDriver()
    {
    }

    @SuppressWarnings("deprecation")
    public Connection connect(String url, Properties info) throws SQLException
    {
        if (!acceptsURL(url))
        {
            throwBadUrlException(url);
        }
        String urlPart = url.substring(10); // Omit "jdbc:db4o:"

        ObjectContainer oc = null;
        if (urlPart.startsWith("file:"))
        {
            // File based URL
            String filename = urlPart.substring(5);
            try
            {
                oc = Db4o.openFile(filename);
                // TODO Use user/password props ?
            }
            catch (DatabaseFileLockedException dbfle)
            {
                throw new SQLException(dbfle.getMessage());
            }
        }
        else
        {
            // Server based URL
            String user = info.getProperty("user");
            String password = info.getProperty("password");
            // try to make connection to db4o server
            String[] urlSplit = url.split("//");
            if (urlSplit.length < 2)
            {
                throwBadUrlException(url);
            }

            // now take second half and get hostname/port
            String[] split2 = urlSplit[1].split(":");
            if (split2.length < 2)
            {
                throwBadUrlException(url);
            }
            String hostname = null;
            int port = 0;
            try
            {
                hostname = split2[0];
                port = Integer.parseInt(split2[1]);
            }
            catch (Exception e)
            {
                throwBadUrlException(url);
            }
            try
            {
                oc = Db4o.openClient(hostname, port, user, password);
            }
            catch (Exception e)
            {
                throw new SQLException(e.getMessage());
            }
        }

        return new Db4oConnection(this, oc);
    }

    private void throwBadUrlException(String url) throws SQLException
    {
        throw new SQLException("JDBC url is not valid [" + url + "].  Must be of the form: jdbc:db4o://HOSTNAME:PORT");
    }

    public boolean acceptsURL(String url) throws SQLException
    {
        return url.startsWith("jdbc:db4o");
    }

    public DriverPropertyInfo[] getPropertyInfo(String url, Properties info) throws SQLException
    {
        return new DriverPropertyInfo[0];
    }

    public int getMajorVersion()
    {
        return VERSION_MAJOR;
    }

    public int getMinorVersion()
    {
        return VERSION_MINOR;
    }

    public boolean jdbcCompliant()
    {
        return false;
    }

    public static String getVersion()
    {
        return VERSION_MAJOR + "." + VERSION_MINOR;
    }

    public static String getDriverName()
    {
        return DRIVER_NAME;
    }
}